(function($, Drupal, drupalSettings) {
  // The function name is prototyped as part of the Drupal.AjaxCommands namespace, adding to the commands:
  Drupal.AjaxCommands.prototype.dkanVisualize = function(ajax, response, status) {
    // The value passed in the Ajax callback function will be available inside the
    // response object.
    let chartStatus = Chart.getChart("dkanChart"); // <canvas> id
    if (chartStatus !== undefined) {
      chartStatus.destroy();
    }


    let datasets = [];
    switch (drupalSettings.chart_definition[drupalSettings.chart_definition.type].chart) {
      case 'bar':
      case 'line':
        for (const [dataKey, dataValue] of Object.entries(drupalSettings.chart_definition.label)) {
          dataValue.forEach((varyKey, i) => buildDatasets(dataKey, varyKey, datasets, drupalSettings));
        }
        break;
      case 'pie':
      case 'doughnut':
        for (const [dataKey, dataValue] of Object.entries(drupalSettings.chart_dataset)) {
          Object.keys(dataValue).forEach((key, i) => datasets.push({
            data: drupalSettings.chart_dataset[dataKey][key],
            label: build_label(dataKey, key)
          }));
        }
        break;
    }
    const ctx = document.getElementById('dkanChart');
    const dkanchart = new Chart(ctx, chartOptions(datasets, drupalSettings));
  };

  function buildDatasets(label, add_label, datasets, drupalSettings) {
    switch (drupalSettings.chart_definition[drupalSettings.chart_definition.type].chart) {
      case 'bar':
      case 'line':
        // remove labels for empty data
        let push = drupalSettings.chart_dataset[label].some(function(obj) {
          return obj[add_label.toString()];
        });
        if (push) {
          datasets.push(
            {
              axis: drupalSettings.chart_definition[drupalSettings.chart_definition.type].axis,
              data: drupalSettings.chart_dataset[label],
              label: build_label(label, add_label),
              borderWidth: 1,
              parsing: {
                [drupalSettings.chart_definition[drupalSettings.chart_definition.type].axiskey.toString() + 'AxisKey']: add_label.toString(),
              }
            },
          );
        }
        break;
    }
  }

  function chartOptions(datasets, drupalSettings) {
    switch (drupalSettings.chart_definition[drupalSettings.chart_definition.type].chart) {
      case 'bar':
      case 'line':
        return {
          type: drupalSettings.chart_definition[drupalSettings.chart_definition.type].chart,
          data: {
            datasets: datasets,
          },
          options: {
            indexAxis: drupalSettings.chart_definition[drupalSettings.chart_definition.type].axis,
            scales: {
              [drupalSettings.chart_definition[drupalSettings.chart_definition.type].axiskey.toString()]: {
                stacked: drupalSettings.chart_definition[drupalSettings.chart_definition.type].stacked
              },
              [drupalSettings.chart_definition[drupalSettings.chart_definition.type].axis.toString()]: {
                stacked: drupalSettings.chart_definition[drupalSettings.chart_definition.type].stacked
              },
            }
          },
        };
        break;
      case 'pie':
      case 'doughnut':
        const autocolors = window['chartjs-plugin-autocolors'];
        return {
          type: drupalSettings.chart_definition[drupalSettings.chart_definition.type].chart,
          data: {
            datasets: datasets,
            labels: drupalSettings.chart_definition.label[Object.keys(drupalSettings.chart_definition.label)[0]],
          },
          plugins: [
            autocolors
          ],
          options: {
            plugins: {
              colors: {
                enabled: false
              },
              autocolors: {
                mode: 'data',
                customize(context) {
                  const colors = context.colors;
                  if (context.datasetIndex > 0) {
                    return {
                      background: context.chart.config._config.data.datasets[0].backgroundColor[context.dataIndex],
                      border: context.chart.config._config.data.datasets[0].borderColor[context.dataIndex],
                    }
                  }
                  return {
                    background: colors.background,
                    border: colors.border
                  };
                }
              }
            }
          }
        };
        break;
    }
  }

  function build_label(label, add_label) {
    if (add_label !== 'default') {
      return label.toString() + ' (' + add_label + ')';
    }
    return label.toString();
  }

}(jQuery, Drupal, drupalSettings));



