<?php

namespace Drupal\dkan_chart\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\dkan_chart\Service\DatastoreVisualizationModeller;
use Drupal\node\NodeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class VisualizeController extends ControllerBase {

  /**
   * @var \Drupal\dkan_chart\Service\DatastoreVisualizationModeller
   */
  protected $datastoreVisualizationModeller;


  /**
   * @param DatastoreVisualizationModeller $datastoreVisualizationModeller
   */
  public function __construct(DatastoreVisualizationModeller $datastoreVisualizationModeller) {
    $this->datastoreVisualizationModeller = $datastoreVisualizationModeller;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('datastore_visualization_modeller')
    );
  }

  /**
   * provide the visualization UI (form)
   *
   * @param NodeInterface $node
   * @param $distribution
   * @return array
   *   render array
   */
  public static function visualizationUi(NodeInterface $node, $distribution = FALSE) {
    $build = [];
    $visualizationModeller = \Drupal::service('datastore_visualization_modeller');
    if ($visualizationModeller instanceof DatastoreVisualizationModeller) {
      $visualizationModeller->setNode($node);
      if (count($visualizationModeller->getDistributionsInDatastore()) === 1 || $distribution) {
        $build['form'] = \Drupal::formBuilder()->getForm('Drupal\dkan_chart\Form\VisualizeAjaxUiForm', $node, $distribution);
        $build['#attached']['library'][] = 'dkan_chart/dkan_chartvisualization';
      }
    }

    return $build;
  }

  /**
   * provide a form for selecting different distributions in datastore
   *
   * @param NodeInterface $node
   * @param $distribution
   * @return array
   *   render array
   */
  public static function visualizationUiDistributionSelection(NodeInterface $node, $distribution = FALSE) {
    $build = [];
    $visualizationModeller = \Drupal::service('datastore_visualization_modeller');
    if ($visualizationModeller instanceof DatastoreVisualizationModeller) {
      $visualizationModeller->setNode($node);
      if (count($visualizationModeller->getDistributionsInDatastore()) > 1) {
        $build['form'] = \Drupal::formBuilder()->getForm('Drupal\dkan_chart\Form\ChooseDistributionForm', $node, $distribution);
      }
    }

    return $build;
  }

  /**
   * check access for visualization UI
   *   granted if
   *   - node access
   *   - node is of type data and data type dataset
   *   - node (dataset) has any distribution in datastore
   *
   * @param AccountInterface $account
   * @param NodeInterface $node
   * @return \Drupal\Core\Access\AccessResultAllowed|\Drupal\Core\Access\AccessResultForbidden
   */
  public function visualizeAccess(AccountInterface $account, NodeInterface $node) {
    if ($node->getType() === 'data' && $node->access('view')) {
      if ($node->hasField('field_data_type') && !$node->get('field_data_type')->isEmpty() && $node->get('field_data_type')->value === 'dataset') {
        $this->datastoreVisualizationModeller->setNode($node);
        if ($this->datastoreVisualizationModeller->getDistributionsInDatastore()) {
          return AccessResult::allowed();
        }
      }
    }
    return AccessResult::forbidden();
  }
}
